/**
 * \file point3d.c
 * \brief A simple implementation of 3D points and 4x4 matrices
 * \author Pascal Getreuer <getreuer@gmail.com>
 */

#include <math.h>
#include "point3d.h"

#ifndef M_PI
/** \brief The constant pi */
#define M_PI        3.14159265358979323846264338327950288
#endif

mat4x4 mat4x4_rot_roll(double angle)
{
    mat4x4 R;
    double c = cos(angle*M_PI/180.0), s = sin(angle*M_PI/180.0);
    R.el[0][0] = c;  R.el[0][1] = s;  R.el[0][2] = 0;  R.el[0][3] = 0;
    R.el[1][0] = -s; R.el[1][1] = c;  R.el[1][2] = 0;  R.el[1][3] = 0;
    R.el[2][0] = 0;  R.el[2][1] = 0;  R.el[2][2] = 1;  R.el[2][3] = 0;
    R.el[3][0] = 0;  R.el[3][1] = 0;  R.el[3][2] = 0;  R.el[3][3] = 1;
    return R;
}

mat4x4 mat4x4_rot_pitch(double angle)
{
    mat4x4 R;
    double c = cos(angle*M_PI/180.0), s = sin(angle*M_PI/180.0);
    R.el[0][0] = 1;  R.el[0][1] = 0;  R.el[0][2] = 0;  R.el[0][3] = 0;
    R.el[1][0] = 0;  R.el[1][1] = c;  R.el[1][2] = s;  R.el[1][3] = 0;
    R.el[2][0] = 0;  R.el[2][1] = -s; R.el[2][2] = c;  R.el[2][3] = 0;
    R.el[3][0] = 0;  R.el[3][1] = 0;  R.el[3][2] = 0;  R.el[3][3] = 1;
    return R;
}

mat4x4 mat4x4_translate(double tx, double ty, double tz)
{
    mat4x4 T;
    T.el[0][0] = 1;  T.el[0][1] = 0;  T.el[0][2] = 0;  T.el[0][3] = tx;
    T.el[1][0] = 0;  T.el[1][1] = 1;  T.el[1][2] = 0;  T.el[1][3] = ty;
    T.el[2][0] = 0;  T.el[2][1] = 0;  T.el[2][2] = 1;  T.el[2][3] = tz;
    T.el[3][0] = 0;  T.el[3][1] = 0;  T.el[3][2] = 0;  T.el[3][3] = 1;
    return T;
}

mat4x4 mat4x4_compose(mat4x4 A, mat4x4 B)
{
    mat4x4 C;
    int i, j, k;
    
    for(i = 0; i < 4; i++)
        for(j = 0; j < 4; j++)
            for(k = 0, C.el[i][j] = 0; k < 4; k++)
                C.el[i][j] += A.el[i][k] * B.el[k][j];
    
    return C;
}

point3d mat4x4_transform_point3d(mat4x4 T, point3d p)
{
    double v[3];
    int i;
    
    for(i = 0; i < 3; i++)
        v[i] = T.el[i][0] * p.x + T.el[i][1] * p.y 
            + T.el[i][2] * p.z + T.el[i][3];
    
    p.x = v[0]; p.y = v[1]; p.z = v[2];
    return p;
}
